@@ -1,6 +1,6 @@
 #include "bios.h"
 
-const char* ffDetectBios(FFBiosResult* bios)
+const char* ffDetectBios(FF_MAYBE_UNUSED FFBiosResult* bios)
 {
     return "Not supported on this platform";
 }
